bool DEBUG = false;

#include <USB-MIDI.h>
USBMIDI_CREATE_DEFAULT_INSTANCE();

#define latchPin 8 //green
#define dataPin 9  //blue
#define clockPin 7 //yellow

#define pressed 0       //set 0 for pullup resistor, 1 for pulldown
#define unpressed 1     //set 1 for pullup resistor, 0 for pulldown
#define numKeys 13       //defines the number of keys on the piano
#define volWindow 300  //defines the length of time a note on signal can be sent after a key is released from the up state

byte switchVar1 = 72;
byte switchVar2 = 89;
byte switchVar3 = 162;
byte switchVar4 = 159;

int dnButton[numKeys];
int upButton[numKeys];
bool noteLatch[numKeys];
bool volTimer[numKeys];
unsigned long startTime[numKeys];
int noteNum[] = {60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72}; //note numbers as listed in MIDI standard (60 = middle C)

//SETUP AND LOOP----------------------------------------------------------------------------------------------------------------------------------

void setup() {
  if (DEBUG == true) {
    Serial.begin(9600);
  }
  else {
    MIDI.begin(MIDI_CHANNEL_OMNI);
    //Serial.begin(9600);
  }
  
  pinMode(latchPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, INPUT);

  for (int i = 0; i < numKeys; i++) { //initialize bool arrays as false
    noteLatch[i] = false;
    volTimer[i] = false;
  }
}

void loop() {
  writeInputs();
  
  for (int i = 0; i < numKeys; i++) {
    keyMonitor(i);
  }
}

//FUNCTIONS----------------------------------------------------------------------------------------------------------------------------------------

void keyMonitor(int keyNum) {
  if (dnButton[keyNum] == pressed) {
    if (noteLatch[keyNum] == false) {
      noteLatch[keyNum] = true;
      unsigned long volume = volWindow - (millis() - startTime[keyNum]);
      if (volume > volWindow) { //if volume underflows to be greater than the volume window, set the volume to 0
        volume = 0;
      }
      volume = map(volume, 0, volWindow, 0, 140); //scale the volume from the volume window to the midi max of 127.
      if (volume > 127) {
        volume = 127;
      }
      //Serial.print(noteNum[keyNum]); Serial.print(" ON, vol: "); Serial.println(volume);
      MIDI.sendNoteOn(noteNum[keyNum], volume, 1);
    }
  }
  else if (upButton[keyNum] == pressed){
    if (noteLatch[keyNum] == true) {
      noteLatch[keyNum] = false;
      volTimer[keyNum] = false;
      //Serial.print(noteNum[keyNum]); Serial.println(" OFF");
      MIDI.sendNoteOff(noteNum[keyNum], 0, 1);
    }
  }
  else if (dnButton[keyNum] == unpressed && upButton[keyNum] == unpressed) {
    if (volTimer[keyNum] == false) {
      volTimer[keyNum] = true;
      startTime[keyNum] = millis();
    }
  }
}

byte shiftIn(int myDataPin, int myClockPin) { //reads data from a single shift register and returns it as a single byte value
  int i;
  int temp = 0;
  int pinState;
  byte myDataIn = 0;

  pinMode(myClockPin, OUTPUT);
  pinMode(myDataPin, INPUT);
  
  for (i=7; i>=0; i--)
  {
    digitalWrite(myClockPin, 0);
    delayMicroseconds(2);
    temp = digitalRead(myDataPin);
    if (temp) {
      pinState = 1;
      myDataIn = myDataIn | (1 << i);
    }
    else {
      pinState = 0;
    }
    digitalWrite(myClockPin, 1);
  }
  return myDataIn;
}

void writeInputs() {
  digitalWrite(latchPin, 1);
  delayMicroseconds(20);
  digitalWrite(latchPin,0);

  switchVar1 = shiftIn(dataPin, clockPin);
  switchVar2 = shiftIn(dataPin, clockPin);
  switchVar3 = shiftIn(dataPin, clockPin);
  switchVar4 = shiftIn(dataPin, clockPin);

  //EDIT INPUTS HERE---------------------------------------------------
  dnButton[0] = bitRead(switchVar3, 7); //ex: Shift register 3, input 8
  dnButton[1] = bitRead(switchVar1, 0);
  dnButton[2] = bitRead(switchVar2, 0);
  dnButton[3] = bitRead(switchVar3, 2);
  dnButton[4] = bitRead(switchVar2, 7);
  dnButton[5] = bitRead(switchVar2, 2);
  dnButton[6] = bitRead(switchVar2, 3);
  dnButton[7] = bitRead(switchVar3, 3);
  dnButton[8] = bitRead(switchVar4, 1);
  dnButton[9] = bitRead(switchVar2, 1);
  dnButton[10] = bitRead(switchVar3, 1);
  dnButton[11] = bitRead(switchVar3, 0);
  dnButton[12] = bitRead(switchVar4, 0);
  upButton[0] = bitRead(switchVar2, 6);
  upButton[1] = bitRead(switchVar1, 5);
  upButton[2] = bitRead(switchVar2, 4);
  upButton[3] = bitRead(switchVar1, 4);
  upButton[4] = bitRead(switchVar3, 6);
  upButton[5] = bitRead(switchVar1, 6);
  upButton[6] = bitRead(switchVar1, 2);
  upButton[7] = bitRead(switchVar3, 4);
  upButton[8] = bitRead(switchVar1, 1);
  upButton[9] = bitRead(switchVar2, 5);
  upButton[10] = bitRead(switchVar1, 7);
  upButton[11] = bitRead(switchVar1, 3);
  upButton[12] = bitRead(switchVar3, 5);
  
  //DEBUG----------------------------------
  if (DEBUG == true) {
    for (int i = 0; i < 8; i++) {
      Serial.print(bitRead(switchVar1, i));
    }
    Serial.print("  ");
    for (int i = 0; i < 8; i++) {
      Serial.print(bitRead(switchVar2, i));
    }
    Serial.print("  ");
    for (int i = 0; i < 8; i++) {
      Serial.print(bitRead(switchVar3, i));
    }
    Serial.print("  ");
    for (int i = 0; i < 2; i++) {
      Serial.print(bitRead(switchVar4, i));
    }
    Serial.println();
  }
}
